// *****************************************************************
//  HDMI in
// *****************************************************************

module HDMI_in_rot #(
parameter int                              PORT_ADDR_SIZE      = 24,   // Must match PORT_ADDR_SIZE.
                                           PIXEL_WIDTH         = 32
)(
input                                      CLK_IN            ,
input                                      CMD_CLK           ,
input                                      reset             ,

input                                      write_busy_in      ,         // DDR3 ram read channel #1 was selected for reading the video ram.


input logic PIXCLK, ACTIVE, HSYNC, VSYNC, DISPEN,
input logic [7:0]                    HDMI_rx_R         ,
input logic [7:0]                    HDMI_rx_G         ,
input logic [7:0]                    HDMI_rx_B         ,

input  logic        [2:0]                  DISP_pixel_bytes   ,         // 4=32 bit pixels, 2=16bit pixels, 1=8bit pixels.
input  logic        [31:0]                 DISP_mem_addr      ,         // Beginning memory address of graphic bitmap pixel position 0x0.
input  logic signed [15:0]                 DISP_bitmap_width  ,         // The bitmap width of the graphic in memory.
input  logic signed [15:0]                 DISP_bitmap_height ,         // The bitmap width of the graphic in memory.

output        logic                        write_req_out      ,
output              [PORT_ADDR_SIZE-1:0]   write_adr_out      ,
output              [31:0]                 write_data_out     ,
//output              [127:0]                 write_data_out     ,
output              [3:0]                  write_mask_out     

);



reg PIXCLKa, ACTIVEa, HSYNCa, VSYNCa, DISPENa		;
reg [7:0]                    HDMI_rx_Ra         ;
reg [7:0]                    HDMI_rx_Ga         ;
reg [7:0]                    HDMI_rx_Ba         ;

always_ff @(posedge PIXCLK ) begin 

PIXCLKa			<= PIXCLK	; 
ACTIVEa        <= ACTIVE;
HSYNCa         <= HSYNC;
VSYNCa         <= VSYNC;
DISPENa        <= DISPEN;
HDMI_rx_Ra     <= HDMI_rx_R;
HDMI_rx_Ga     <= HDMI_rx_G;
HDMI_rx_Ba     <= HDMI_rx_B;
end

//logic [31:0] counter = 0 ;
logic [15:0] PIXEL_counter = 0 ;
logic [15:0] LINE_counter = 0 ;

logic [1:0]                 pixel_byte_shift ;

reg [2:0] VSYNCr;
always @(posedge PIXCLK) VSYNCr <= {VSYNCr[1:0], VSYNCa};
wire VSYNC_risingedge = (VSYNCr[2:1] == 2'b01);
wire VSYNC_fallingedge = (VSYNCr[2:1] == 2'b10);
wire VSYNC_active = ~VSYNCr[1];

reg [2:0] HSYNCr;
always @(posedge PIXCLK) HSYNCr <= {HSYNCr[1:0], HSYNCa};
wire HSYNC_risingedge = (HSYNCr[2:1] == 2'b01);
wire HSYNC_fallingedge = (HSYNCr[2:1] == 2'b10);
wire HSYNC_active = ~HSYNCr[1];



//assign write_adr_out = write_adr;
assign write_mask_out = 4'b1111 ; // Always write enable all bits.


// **********************************************************************************
// Generate a write pixel cache port.
// **********************************************************************************
logic [PORT_ADDR_SIZE-1:0]   write_adr  ;
logic [31:0]                 write_data ;
logic read_req_cache,write_req_cache,pixel_cache_busy,pixel_cache_empty,pixel_cache_full;

    scfifo    write_pixel_cache (
                .clock        ( CMD_CLK                        ),
                .data         ( {write_adr    ,color    } ),
                .rdreq        ( read_req_cache                 ),
                .wrreq        ( write_req_cache                ),
//                .wrreq        ( PIXCLK                ),
                .almost_full  ( pixel_cache_busy               ),
                .empty        ( pixel_cache_empty              ),
                .full         ( pixel_cache_full               ),
                .q            ( {write_adr_out,write_data_out} ),
                .aclr         ( reset                          ),
                .almost_empty (),.eccstatus (),.sclr (),.usedw ());
    defparam
        write_pixel_cache.add_ram_output_register = "ON",
        write_pixel_cache.almost_full_value = 1000,
        write_pixel_cache.intended_device_family = "MAX 10",
        write_pixel_cache.lpm_numwords = 1024,
        write_pixel_cache.lpm_showahead = "ON",//"OFF",
        write_pixel_cache.lpm_type = "scfifo",
        write_pixel_cache.lpm_width = (PORT_ADDR_SIZE+PIXEL_WIDTH),
        write_pixel_cache.lpm_widthu = 10,
        write_pixel_cache.overflow_checking = "ON",
        write_pixel_cache.underflow_checking = "ON",
        write_pixel_cache.use_eab = "ON";

// **********************************************************************************
// Manage pixel write cache FIFO to DDR3 write port out.
// **********************************************************************************
always_comb read_req_cache = (!pixel_cache_empty && !write_busy_in) ;
always_comb write_req_out  = (!pixel_cache_empty && !write_busy_in) ;



//logic               [127:0] color=0,xycol1=0,xycol2=0,xycol3=0;
logic               [31:0] color=0,xycol1=0,xycol2=0,xycol3=0;
logic                      xyr1=0,xyr2=0,xyr3=0,pixel_in_pipe,sel_draw=0,draw_ena=0,rnd_sel=0;
logic signed        [13:0] x1=0,x2=0,x3=0,y1=0,y2=0,draw_x=0,draw_y=0;
logic [PORT_ADDR_SIZE-1:0] addr1=0;




always_ff @(posedge PIXCLK ) begin 

	if (VSYNC_fallingedge) begin
		PIXEL_counter         <= 0 ;
		LINE_counter         <= 0 ;
	end 
		
	else if (HSYNC_fallingedge) begin
		PIXEL_counter         <= 0 ;
		LINE_counter         <= LINE_counter + 1 ;
	end

	 
//	else if (HSYNCa == 0 & VSYNCa == 0 & ACTIVEa == 1) 
	else if (HSYNCa == 0 & VSYNCa == 0 & DISPENa == 1) 

	 begin  
		PIXEL_counter    <= PIXEL_counter + 1 ; 
		color[31:0]  <= 	{8'h00, HDMI_rx_Ba,HDMI_rx_Ga,HDMI_rx_Ra};  //32bit
//		color[31:0]  <= 	{8'h00, 8'h00,8'hff,8'h00,};  //32bit  green

	end
end

//endmodule


// **********************************************************************************
// Convert X & Y output coordinates into a memory address.
// Step 1, Select source drawing data pipe, ELLI or other...
// Step 2, make sure the coordinates are inside the display area.
// Step 3, compute the base Y coordinate address offset
// Step 4, Add the X position coordinate.
// All these steps must sync & hold data to the 'elli_out_rdy' and 'pixel_cache_busy' flags.
// 
// **********************************************************************************

always_comb  pixel_in_pipe = xyr1 || xyr2 || xyr3 ; // make a status flag indicating that there are active pixels in the processing pipe. 
always_ff @(posedge CMD_CLK) begin 
if (reset) begin
    xyr1              <= 0 ;
    xyr2              <= 0 ;
    xyr3              <= 0 ;
    write_req_cache   <= 0 ;
end else begin

// Step 1, select between elli output and other graphics generators.
//    xyr1   <=  draw_ena ;
//    x1     <=  draw_x   ;
//    y1     <=  draw_y   ;
	 xyr1   <=  DISPENa	;
//    xyr1   <=  1 ;
    y1     <=  PIXEL_counter   ; //SWAP to rotate
    x1     <=  LINE_counter   ;	//SWAP to rotate
    xycol1 <= color        ;

// Step 2, make sure the coordinates are inside the display area.
    if (x1>=0 && x1<DISP_bitmap_width && y1>=0 && y1<=DISP_bitmap_height)  xyr2 <= xyr1 ; // If draw coordinates are inside the bitmap, allow the pixel ready through.
    else                                                                   xyr2 <= 0    ; // otherwise, strip out the
    x2     <= x1      ;
    y2     <= y1      ;
    xycol2 <= xycol1  ;

// Step 3, compute the base Y coordinate address offset
    xyr3   <= xyr2 ;
    addr1  <= PORT_ADDR_SIZE'(DISP_mem_addr + (y2*DISP_bitmap_width)) ;
    x3     <= x2      ;
    xycol3 <= xycol2  ;
    
// Step 4, Add the X position coordinate and output to pixel write fifo
    write_req_cache   <= xyr3 ;
    write_adr         <= (addr1 + x3) << pixel_byte_shift ;
    write_data        <= xycol3  ;

  end // !reset
end // always


// **********************************************************************************
// Translate inputs.
// **********************************************************************************
always_comb begin

    // Convert the pixel width input into a bit shift for addressing pixels.
    case (DISP_pixel_bytes) 
            2 : pixel_byte_shift = 1 ;
            4 : pixel_byte_shift = 2 ;
      default : pixel_byte_shift = 0 ;
    endcase

end // always comb...

endmodule

